@@ -24,29 +24,26 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <libintl.h>
 #include <hildon/hildon-program.h>
 #include <hildon/hildon-banner.h>
 #include <hildon/hildon-defines.h>
-/*#include <libhildondesktop/hildon-thumb-menu-item.h>*/
-#include <glib/gstdio.h>
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
 #include <gdk/gdkkeysyms.h>
 #include <math.h>
 
 #include "callbacks.h"
-
 #include "metadata.h"
 #include "interface.h"
-
 #include "note.h"
 #include "serializer.h"
 #include "deserializer.h"
 #include "search_window.h"
+#include "note_linker.h"
+#include "app_data.h"
 
-#ifndef max
-	#define max( a, b ) ( ((a) > (b)) ? (a) : (b) )
-#endif
+#define _(String)gettext(String)
 
 
 /* Private. TODO: Move to some public file */
@@ -180,11 +177,12 @@ on_font_size_radio_group_changed       (GtkRadioAction  *action,
 
 
 gboolean
-on_window_close_button_clicked		   (GtkObject		*window,
+on_window_delete		               (GtkObject		*window,
 										GdkEvent		*event,
 										gpointer		 user_data)
 {
 	Note *note = (Note*)user_data;
+	g_printerr("Window delete \n");
 	note_save(note);
 	note_close_window(note);
 	return TRUE; /* True to stop other handler from being invoked */
@@ -193,7 +191,7 @@ on_window_close_button_clicked		   (GtkObject		*window,
 void on_quit_button_clicked(GtkAction *action, gpointer user_data)
 {
 	Note *note;
-	GList *open_notes = get_app_data()->open_notes;
+	GList *open_notes = app_data_get()->open_notes;
 
 	while (open_notes != NULL) {
 		note = (Note*)open_notes->data;
@@ -389,7 +387,7 @@ on_notes_button_clicked				   (GtkAction		*action,
 	GTimer *timer;
 	gulong micro;
 
-	app_data = get_app_data();
+	app_data = app_data_get();
 	if (app_data->note_store == NULL) {
 		return;
 	}
@@ -472,6 +470,7 @@ on_textview_cursor_moved			   (GtkTextBuffer	*buffer,
 	gtk_toggle_action_set_active(ui->action_highlight, FALSE);
 	gtk_toggle_action_set_active(ui->action_fixed, FALSE);
 	gtk_toggle_action_set_active(ui->action_bullets, FALSE);
+	gtk_radio_action_set_current_value(ui->action_font_small, 1); /* Enable normal font size */
 
 	while (tags != NULL) {
 		tag = GTK_TEXT_TAG(tags->data);
@@ -489,8 +488,6 @@ on_textview_cursor_moved			   (GtkTextBuffer	*buffer,
 			gtk_toggle_action_set_active(ui->action_bullets, TRUE);
 		} else if (strcmp(tag->name, "size:small") == 0) {
 			gtk_radio_action_set_current_value(ui->action_font_small, 0);
-		} else if (strcmp(tag->name, "size:normal") == 0) {
-			gtk_radio_action_set_current_value(ui->action_font_small, 1);
 		} else if (strcmp(tag->name, "size:large") == 0) {
 			gtk_radio_action_set_current_value(ui->action_font_small, 2);
 		} else if (strcmp(tag->name, "size:huge") == 0) {
@@ -547,7 +544,7 @@ on_link_internal_tag_event				(GtkTextTag  *tag,
 										 gpointer     user_data)
 {
 	GdkEventType type = event->type;
-	GtkTextIter *start;
+	GtkTextIter start;
 	gchar *link_text;
 
 	if (type == GDK_BUTTON_RELEASE) {
@@ -559,13 +556,13 @@ on_link_internal_tag_event				(GtkTextTag  *tag,
 				gtk_text_iter_backward_to_tag_toggle(iter, tag);
 			}
 
-			start = gtk_text_iter_copy(iter);
+			start = *iter;
 
 			if (!gtk_text_iter_ends_tag(iter, tag)) {
 				gtk_text_iter_forward_to_tag_toggle(iter, tag);
 			}
 
-			link_text = gtk_text_iter_get_text(start, iter);
+			link_text = gtk_text_iter_get_text(&start, iter);
 
 			g_printerr("Link: >%s< \n", link_text);
 			note_show_by_title(link_text);
@@ -579,46 +576,33 @@ void
 on_delete_button_clicked			   (GtkAction		*action,
 										gpointer		 user_data)
 {
-	/* Popup Dialog ask for sure
-	 * Close window
-	 * Lower window count
-	 * Change open_windows in app_data
-	 * Delete file
-	 * Free resources
-	 */
 	Note *note = (Note*)user_data;
-	const gchar *message = "<b>Really delete this note?</b>\n\n If you delete a note it is permanently lost.";
-	GtkWidget *dialog;
-	gint response;
-	AppData *app_data;
 
-	dialog = gtk_message_dialog_new_with_markup(
+	gchar *message = g_strconcat("<b>",
+			_("Really delete this note?"),
+			"</b>\n\n",
+			_("If you delete a note it is permanently lost."),
+			NULL);
+
+	GtkWidget *dialog = gtk_message_dialog_new_with_markup(
 			GTK_WINDOW(note->ui->window),
 			GTK_DIALOG_MODAL,
 			GTK_MESSAGE_QUESTION,
 			GTK_BUTTONS_YES_NO,
 			message);
 
-	response = gtk_dialog_run(GTK_DIALOG(dialog));
+	gint response = gtk_dialog_run(GTK_DIALOG(dialog));
+	
 	gtk_widget_destroy(dialog);
 
-	if (response != GTK_RESPONSE_YES) {
-		return;
+	if (response == GTK_RESPONSE_YES) {
+		/* Delete note and close window */
+		note_delete(note);
+		note_close_window(note);
+		note_free(note);
 	}
 
-	/* Block this signal, otherwise the destroy event of the window will trigger saving of the note */
-	g_signal_handlers_block_by_func(note->ui->window, on_window_close_button_clicked, note);
-	note_close_window(note);
-	/* Don't unblock the signal, because the window does not exist anymore */
-
-	/* Delete file */
-	if (g_unlink(note->filename) == -1) {
-		g_printerr("ERROR: The file %s could not be deleted \n", note->filename);
-	}
-
-	app_data = get_app_data();
-	/* TODO: Free note */
-	note_list_store_remove(app_data->note_store, note);
+	g_free(message);
 }
 
 static
@@ -653,7 +637,7 @@ on_text_buffer_modified_changed			(GtkTextBuffer *buffer,
 static
 void change_font_size_by(gint size)
 {
-	AppData *app_data = get_app_data();
+	AppData *app_data = app_data_get();
 	PangoFontDescription *font;
 	Note *note;
 	GList *note_list;
@@ -709,7 +693,7 @@ gboolean on_hardware_key_pressed	(GtkWidget			*widget,
 {
 	Note *note = (Note*)user_data;
 	GtkWidget *window = GTK_WIDGET(note->ui->window);
-	AppData *app_data = get_app_data();
+	AppData *app_data = app_data_get();
 	GList *open_notes;
 
 	switch (event->keyval) {
@@ -724,7 +708,7 @@ gboolean on_hardware_key_pressed	(GtkWidget			*widget,
 		return TRUE;
 
 	case HILDON_HARDKEY_ESC:
-		on_window_close_button_clicked(GTK_OBJECT(window), NULL, note);
+		on_window_delete(GTK_OBJECT(window), NULL, note);
 		return TRUE;
 
 	case HILDON_HARDKEY_FULLSCREEN:
@@ -742,6 +726,14 @@ gboolean on_hardware_key_pressed	(GtkWidget			*widget,
 			}
 			open_notes = open_notes->next;
 		}
+		/* Set search window to fullscreen or unfullscreen */
+		if (app_data->search_window != NULL) {
+			if (app_data->fullscreen) {
+				gtk_window_fullscreen(GTK_WINDOW(app_data->search_window));
+			} else {
+				gtk_window_unfullscreen(GTK_WINDOW(app_data->search_window));
+			}
+		}
 
 		/* Focus again on the active note */
 		note_set_focus(note);
@@ -837,7 +829,7 @@ static gboolean add_new_line(Note *note)
 		} else {
 			/* Remove bullet and insert newline */
 			GtkTextTag *tag;
-			GtkTextIter *start = gtk_text_iter_copy(&iter);
+			GtkTextIter start = iter;
 
 			/* Disable list and list-item tags */
 			tag = gtk_text_tag_table_lookup(buffer->tag_table, "list-item");
@@ -846,10 +838,10 @@ static gboolean add_new_line(Note *note)
 			note_remove_active_tag(note, tag);
 
 			/* Delete the bullet and the last newline */
-			gtk_text_iter_set_line_offset(start, 0);
-			gtk_text_iter_backward_char(start);
-			gtk_text_buffer_remove_all_tags(buffer, start, &iter);
-			gtk_text_buffer_delete(buffer, start, &iter);
+			gtk_text_iter_set_line_offset(&start, 0);
+			gtk_text_iter_backward_char(&start);
+			gtk_text_buffer_remove_all_tags(buffer, &start, &iter);
+			gtk_text_buffer_delete(buffer, &start, &iter);
 
 			gtk_text_buffer_insert(buffer, &iter, "\n", -1);
 
@@ -864,21 +856,21 @@ static gboolean add_new_line(Note *note)
 		/* If line start with a "- " or "- *" */
 		if (line_needs_bullet(buffer, gtk_text_iter_get_line(&iter))) {
 			GSList *tmp;
-			GtkTextIter *end_iter;
+			GtkTextIter end_iter;
 
 			gtk_text_iter_set_line_offset(&iter, 0);
-			end_iter = gtk_text_iter_copy(&iter);
+			end_iter = iter;
 			line = gtk_text_iter_get_line(&iter);
 
 			/* Skip trailing spaces */
-			while (gtk_text_iter_get_char(end_iter) == ' ') {
-				gtk_text_iter_forward_char(end_iter);
+			while (gtk_text_iter_get_char(&end_iter) == ' ') {
+				gtk_text_iter_forward_char(&end_iter);
 			}
 			/* Skip "* " or "- " */
-			gtk_text_iter_forward_chars(end_iter, 2);
+			gtk_text_iter_forward_chars(&end_iter, 2);
 
 			/* Delete this part */
-			gtk_text_buffer_delete(buffer, &iter, end_iter);
+			gtk_text_buffer_delete(buffer, &iter, &end_iter);
 
 			/* Add bullet for this line */
 			add_bullets(buffer, line, line, buffer_get_depth_tag(buffer, 1));
@@ -934,179 +926,10 @@ on_text_view_key_pressed                      (GtkWidget   *widget,
 
 }
 
-/* TODO: Optimize this. We need a place where all titles are stored, info which is the longest, ... */
-static gint
-get_length_of_longest_title()
-{
-	gint max_length = 0;
-	AppData *app_data = get_app_data();
-	GtkTreeIter iter;
-	GtkTreeModel *model = GTK_TREE_MODEL(app_data->note_store);
-	gboolean valid = gtk_tree_model_get_iter_first(model, &iter);
-
-	while (valid) {
-		Note *note;
-		gtk_tree_model_get(model, &iter, NOTE_COLUMN, &note, -1);
-		max_length = max(g_utf8_strlen(note->title, -1), max_length);
-		valid = gtk_tree_model_iter_next(model, &iter); /* TODO: Too much casting in this function. Maybe introduce note_list_store_iter_next() etc... */
-	}
-
-	return max_length;
-}
-
-typedef struct {
-	gint   start_offset;
-	gint   end_offset;
-	Note  *note;
-} SearchHit;
-
-static
-GSList* find_titles(gchar *haystack) {
-	/* TODO: The search algorithm needs to be optimized. Probably to use Aho-Corasick */
-	/* ATM On the device (N810) searching takes around 1500 microseconds with ~70 notes. */
-
-	GTimer *search_timer = g_timer_new();
-
-
-	gchar *found;
-	GSList *result = NULL;
-	AppData *app_data = get_app_data();
-
-	gchar *u_haystack = g_utf8_casefold(haystack, -1);
-	gulong micro;
-
-	GtkTreeIter iter;
-	GtkTreeModel *model = GTK_TREE_MODEL(app_data->note_store);
-	gboolean valid = gtk_tree_model_get_iter_first(model, &iter);
-	while (valid) {
-		gchar *u_needle;
-		Note *note;
-		gtk_tree_model_get(model, &iter, NOTE_COLUMN, &note, -1);
-
-		u_needle = g_utf8_casefold(note->title, -1);
-
-		found = u_haystack;
-
-		while ( (found = strstr(found, u_needle)) != NULL ) {
-			SearchHit *hit = g_new0(SearchHit, 1);
-			hit->note = note;
-			hit->start_offset = g_utf8_pointer_to_offset(u_haystack, found);
-			hit->end_offset = hit->start_offset + g_utf8_strlen(u_needle, -1);
-			result = g_slist_prepend(result, hit);
-			found = found + strlen(u_needle);
-		}
-		g_free(u_needle);
-		valid = gtk_tree_model_iter_next(model, &iter);
-	}
-
-	g_timer_stop(search_timer);
-
-	g_timer_elapsed(search_timer, &micro);
-	/*g_printerr("Search took %lu microseconds \n", micro);*/
-
-	g_free(u_haystack);
-
-	return result;
-}
-
-static void
-extend_block(GtkTextIter *start_iter, GtkTextIter *end_iter, gint max_len, GtkTextTag *tag)
-{
-	/* Set start_iter max_len chars to the left or to the start of the line */
-	if (gtk_text_iter_get_line_offset(start_iter) - max_len > 0) {
-		gtk_text_iter_backward_chars(start_iter, max_len);
-	} else {
-		gtk_text_iter_set_line_offset(start_iter, 0);
-	}
-
-	/* Set end_iter max_len chars to the right or to the end of the line */
-	if (gtk_text_iter_get_line_offset(end_iter) + max_len < gtk_text_iter_get_chars_in_line(end_iter) ) {
-		gtk_text_iter_forward_chars(end_iter, max_len);
-	} else {
-		gtk_text_iter_forward_to_line_end(end_iter);
-	}
-
-	/* Expand selection to the left, if there is a link_tag inside */
-	if (gtk_text_iter_has_tag(start_iter, tag)) {
-		gtk_text_iter_backward_to_tag_toggle(start_iter, tag);
-	}
-
-	/* Expand selection to the right, if there is a link_tag inside */
-	if (gtk_text_iter_has_tag(end_iter, tag)) {
-		gtk_text_iter_forward_to_tag_toggle(end_iter, tag);
-	}
-}
-
-static void
-highlight_titles(Note *note, GtkTextBuffer *buffer, GtkTextIter *start_iter, GtkTextIter *end_iter)
-{
-	GtkTextTag *url_tag  = gtk_text_tag_table_lookup(buffer->tag_table, "link:url");
-	/* For all titles look if they occure in haystack. For all matches apply tag */
-	GSList *hits = find_titles(gtk_text_buffer_get_slice(buffer, start_iter, end_iter, FALSE));
-	while (hits != NULL) {
-		GtkTextIter *hit_start, *hit_end;
-		SearchHit *hit = (SearchHit*)hits->data;
-
-		hit_start = gtk_text_iter_copy(start_iter);
-		gtk_text_iter_forward_chars(hit_start, hit->start_offset);
-
-		hit_end = gtk_text_iter_copy(start_iter);
-		gtk_text_iter_forward_chars(hit_end, hit->end_offset);
-
-		/* Only link agains words or sentencens */
-		if ( (!gtk_text_iter_starts_word(hit_start) && !gtk_text_iter_starts_sentence(hit_start)) ||
-				(!gtk_text_iter_ends_word(hit_end) && !gtk_text_iter_ends_sentence(hit_end))) {
-			hits = hits->next;
-			continue;
-		}
-
-		/* Don´t link against the note itself */
-		if (hit->note == note) {
-			hits = hits->next;
-			continue;
-		}
-
-		/* Don't create links inside external links */
-		if (gtk_text_iter_has_tag(hit_start, url_tag)) {
-			hits = hits->next;
-			continue;
-		}
-
-		/* Apply the tag */
-		gtk_text_buffer_apply_tag_by_name(buffer, "link:internal", hit_start, hit_end);
-
-		hits = hits->next;
-	}
-
-	g_slist_free(hits);
-}
-
-static void
-auto_highlight_links(GtkTextBuffer *buffer, GtkTextIter *iter, const gchar *input, Note *note)
-{
-	GtkTextIter *start_iter, *end_iter;
-	GtkTextTag *link_tag = gtk_text_tag_table_lookup(buffer->tag_table, "link:internal");
-	int max_len = get_length_of_longest_title();
-
-	start_iter = gtk_text_iter_copy(iter);
-	end_iter = gtk_text_iter_copy(iter);
-
-	/* Move start iter back to the position before the insert */
-	gtk_text_iter_backward_chars(start_iter, g_utf8_strlen(input, -1));
-
-	extend_block(start_iter, end_iter, max_len, link_tag);
-
-	/* Remove existing link tag */
-	gtk_text_buffer_remove_tag(buffer, link_tag, start_iter, end_iter);
-
-	/* Add links */
-	highlight_titles(note, buffer, start_iter, end_iter);
-}
-
 static void
 apply_active_tags(GtkTextBuffer *buffer, GtkTextIter *iter, const gchar *input, Note *note)
 {
-	GtkTextIter *start_iter;
+	GtkTextIter start_iter;
 	GSList *active_tags = note->active_tags;
 
 	/* Only apply active tags on typed text, not on pasted text */
@@ -1120,12 +943,12 @@ apply_active_tags(GtkTextBuffer *buffer, GtkTextIter *iter, const gchar *input,
 	}
 
 	/* First remove all tags, then apply all active tags */
-	start_iter = gtk_text_iter_copy(iter);
-	gtk_text_iter_backward_chars(start_iter, g_utf8_strlen(input, -1));
-	gtk_text_buffer_remove_all_tags(buffer, start_iter, iter);
+	start_iter = *iter;
+	gtk_text_iter_backward_chars(&start_iter, g_utf8_strlen(input, -1));
+	gtk_text_buffer_remove_all_tags(buffer, &start_iter, iter);
 
 	while (active_tags != NULL && active_tags->data != NULL) {
-		gtk_text_buffer_apply_tag(buffer, active_tags->data, start_iter, iter);
+		gtk_text_buffer_apply_tag(buffer, active_tags->data, &start_iter, iter);
 		active_tags = active_tags->next;
 	}
 
@@ -1139,6 +962,7 @@ on_text_buffer_insert_text					(GtkTextBuffer *buffer,
 											 gpointer		user_data)
 {
 	Note *note = (Note*)user_data;
+	GtkTextIter start_iter, end_iter;
 	GTimer *timer;
 	gulong micro;
 
@@ -1151,7 +975,13 @@ on_text_buffer_insert_text					(GtkTextBuffer *buffer,
 
 	timer = g_timer_new();
 
-	auto_highlight_links(buffer, iter, text, note);
+	start_iter = *iter;
+	end_iter = *iter;
+
+	/* Move start iter back to the position before the insert */
+	gtk_text_iter_backward_chars(&start_iter, g_utf8_strlen(text, -1));
+	
+	auto_highlight_links(note, &start_iter, &end_iter);
 
 	g_timer_stop(timer);
 	g_timer_elapsed(timer, &micro);
@@ -1165,25 +995,37 @@ on_text_buffer_delete_range					(GtkTextBuffer *buffer,
 											 gpointer		user_data)
 {
 	Note *note = (Note*)user_data;
+	/*
 	GtkTextTag *link_tag;
 	gint max_len;
+	*/
 
 	/* Don't do anything when in the title line */
 	if (gtk_text_iter_get_line(start_iter) == 0 || gtk_text_iter_get_line(end_iter) == 0) {
 		return;
 	}
 
+	auto_highlight_links(note, start_iter, end_iter);
+	
+	/*
 	link_tag = gtk_text_tag_table_lookup(buffer->tag_table, "link:internal");
 	max_len = get_length_of_longest_title();
+	*/
 
 	/* This handler runs after the default handler, so we can modify the iters as we like */
+	/*
 	extend_block(start_iter, end_iter, max_len, link_tag);
+	*/
 
 	/* Remove link tags */
+	/*
 	gtk_text_buffer_remove_tag(buffer, link_tag, start_iter, end_iter);
+	*/
 
 	/* Add link tags */
+	/*
 	highlight_titles(note, buffer, start_iter, end_iter);
+	*/
 }
 
 /*
