@@ -15,12 +15,16 @@
  * You should have received a copy of the GNU General Public License
  * along with Conboy. If not, see <http://www.gnu.org/licenses/>.
  */
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <string.h>
 #include <stdio.h>
+#include <libintl.h>
 
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
@@ -28,6 +32,7 @@
 #include <hildon/hildon-find-toolbar.h>
 #ifdef HILDON_HAS_APP_MENU
 #include <hildon/hildon-text-view.h>
+#include <hildon/hildon-pannable-area.h>
 #endif
 
 #include "callbacks.h"
@@ -36,7 +41,10 @@
 #include "deserializer.h"
 #include "metadata.h"
 #include "interface.h"
+#include "app_data.h"
 
+#define _(String)gettext(String)
+/*
 #define ACCEL_PATH_ROOT            "<NOTE_WINDOW>"
 #define ACCEL_PATH_QUIT            ACCEL_PATH_ROOT"/Quit"
 #define ACCEL_PATH_NEW             ACCEL_PATH_ROOT"/New"
@@ -48,13 +56,16 @@
 #define ACCEL_PATH_STYLE_STRIKE    ACCEL_PATH_STYLE"/Strike"
 #define ACCEL_PATH_STYLE_HIGHLIGHT ACCEL_PATH_STYLE"/Highlight"
 #define ACCEL_PATH_FIND            ACCEL_PATH_ROOT"/Find"
+*/
 
+/*
 static void set_tool_button_icon_by_name(GtkToolButton *button, const gchar *icon_name)
 {
 	GtkWidget *image;
 	image = gtk_image_new_from_icon_name(icon_name, GTK_ICON_SIZE_SMALL_TOOLBAR);
 	gtk_tool_button_set_icon_widget(button, image);
 }
+*/
 
 static void initialize_tags(GtkTextBuffer *buffer) {
 	/*
@@ -91,42 +102,38 @@ static void initialize_tags(GtkTextBuffer *buffer) {
 	*/
 }
 
-/* TODO: Put in util file, it's used in callbacks.c too */
-static
-void set_menu_item_label(GtkMenuItem *item, const gchar *text)
+static void
+set_item_label(GtkContainer *item, const gchar *open_tag, const gchar *text, const gchar *end_tag)
 {
-	GList *children = gtk_container_get_children(GTK_CONTAINER(item));
+	GList *children = gtk_container_get_children(item);
+	gchar *string;
 
 	if (children == NULL) {
-		g_printerr("ERROR: set_menu_item_label() expects a GtkMenuItem, which already contains a label.\n");
+		g_printerr("ERROR: set_menu_item_label() expects a GtkContainer, which contains one label.\n");
 		return;
 	}
 
-	gtk_label_set_markup(GTK_LABEL(children->data), text);
+	string = g_strconcat(open_tag, text, end_tag, NULL);
+
+	/* TODO: Add checks, that it is really a label and the widget is really a Button or a MenuItem */
+	gtk_label_set_markup(GTK_LABEL(children->data), string);
+
+	g_free(string);
 }
 
-static
-void set_button_label(GtkMenuItem *item, const gchar *text)
-{
-	GList *children = gtk_container_get_children(GTK_CONTAINER(item));
-
-	if (children == NULL) {
-		g_printerr("ERROR: set_menu_item_label() expects a GtkMenuItem, which already contains a label.\n");
-		return;
-	}
-
-	gtk_label_set_markup(GTK_LABEL(children->data), text);
-}
-
-
+/*
+ * I'm not sure what is better:
+ * 1) Declaring all as GtkWidget, so that Fremantle and Diablo code can share these.
+ * 2) Declare all as the actual types, so inside the code it's easy to know which type a widget has.
+ */
 GtkWidget* create_mainwin(Note *note) {
 
 	UserInterface *ui = note->ui;
 
 	GtkWidget *mainwin;
 	GtkWidget *vbox1;
-	GtkMenuShell *main_menu;
-	GtkMenuShell *text_style_menu;
+	GtkWidget *main_menu;
+	GtkWidget *text_style_menu;
 
 	GtkWidget *menu_new;
 	GtkWidget *menu_bold;
@@ -151,13 +158,13 @@ GtkWidget* create_mainwin(Note *note) {
 	GtkWidget *textview;
 	GtkTextBuffer *buffer;
 
-	GtkToolItem *button_link;
-	GtkToolItem *button_delete;
-	GtkToolItem *button_notes;
-	GtkToolItem *button_inc_indent;
-	GtkToolItem *button_dec_indent;
-	GtkToolItem *button_style;
-	GtkToolItem *button_find;
+	GtkWidget *button_link;
+	GtkWidget *button_delete;
+	GtkWidget *button_notes;
+	GtkWidget *button_inc_indent;
+	GtkWidget *button_dec_indent;
+	GtkWidget *button_style;
+	GtkWidget *button_find;
 
 	GtkAction *action_new;
 	GtkAction *action_delete;
@@ -181,114 +188,115 @@ GtkWidget* create_mainwin(Note *note) {
 	GtkAction *action_font_huge;
 	GtkAction *action_find;
 
+	GtkActionGroup *action_group;
+
 	GtkTextTag *link_internal_tag;
 
-	GSList *action_group = NULL;
+	GSList *radio_group = NULL;
 
 	GtkAccelGroup *accel_group;
 
 	PangoFontDescription *font;
-	AppData *app_data = get_app_data();
+	AppData *app_data = app_data_get();
 
 	mainwin = hildon_window_new();
-	gtk_window_set_title(GTK_WINDOW(mainwin), ("Conboy"));
+	gtk_window_set_title(GTK_WINDOW(mainwin), "Conboy");
 
 	accel_group = gtk_accel_group_new();
 	gtk_window_add_accel_group(GTK_WINDOW(mainwin), accel_group);
+	g_object_unref(accel_group);
 
 	vbox1 = gtk_vbox_new(FALSE, 0);
 	gtk_widget_show(vbox1);
 	gtk_container_add(GTK_CONTAINER(mainwin), vbox1);
 
 	/* ACTIONS */
-	action_bold = GTK_ACTION(gtk_toggle_action_new("bold", "Bold", NULL, GTK_STOCK_BOLD));
-	action_bullets = GTK_ACTION(gtk_toggle_action_new("bullets", "Bullets", NULL, NULL));
-	action_dec_indent = GTK_ACTION(gtk_action_new("dec_indent", "Decrease Indent", NULL, GTK_STOCK_UNINDENT));
-	action_delete = GTK_ACTION(gtk_action_new("delete", "Delete Note", NULL, GTK_STOCK_DELETE));
-	action_fixed = GTK_ACTION(gtk_toggle_action_new("monospace", "Fixed Width", NULL, NULL));
-	action_highlight = GTK_ACTION(gtk_toggle_action_new("highlight", "Highlight", NULL, NULL));
-	action_inc_indent = GTK_ACTION(gtk_action_new("inc_indent", "Increase Indent", NULL, GTK_STOCK_INDENT));
-	action_link = GTK_ACTION(gtk_action_new("link", "Link", NULL, GTK_STOCK_REDO));
-	action_new = GTK_ACTION(gtk_action_new("new", "New Note", NULL, NULL));
-	action_notes = GTK_ACTION(gtk_action_new("open", "Open", NULL, GTK_STOCK_OPEN));
-	action_quit = GTK_ACTION(gtk_action_new("quit", "Close All Notes", NULL, NULL));
-	action_italic = GTK_ACTION(gtk_toggle_action_new("italic", "Italic", NULL, GTK_STOCK_ITALIC));
-	action_strike = GTK_ACTION(gtk_toggle_action_new("strikethrough", "Strikeout", NULL, GTK_STOCK_STRIKETHROUGH));
-	action_text_style = GTK_ACTION(gtk_action_new("style", "Style", NULL, GTK_STOCK_SELECT_FONT));
-	action_zoom_in = GTK_ACTION(gtk_action_new("zoom_in", "Zoom In", NULL, GTK_STOCK_ZOOM_IN));
-	action_zoom_out = GTK_ACTION(gtk_action_new("zoom_out", "Zoom Out", NULL, GTK_STOCK_ZOOM_OUT));
-	action_find = GTK_ACTION(gtk_action_new("find", "Find In Note", NULL, GTK_STOCK_FIND));
+	action_bold = GTK_ACTION(gtk_toggle_action_new("bold", _("Bold"), NULL, NULL));
+	action_bullets = GTK_ACTION(gtk_toggle_action_new("bullets", _("Bullets"), NULL, NULL));
+	action_dec_indent = GTK_ACTION(gtk_action_new("dec_indent", _("Decrease Indent"), NULL, GTK_STOCK_UNINDENT));
+	action_delete = GTK_ACTION(gtk_action_new("delete", _("Delete Note"), NULL, GTK_STOCK_DELETE));
+	action_fixed = GTK_ACTION(gtk_toggle_action_new("monospace", _("Fixed Width"), NULL, NULL));
+	action_highlight = GTK_ACTION(gtk_toggle_action_new("highlight", _("Highlight"), NULL, NULL));
+	action_inc_indent = GTK_ACTION(gtk_action_new("inc_indent", _("Increase Indent"), NULL, GTK_STOCK_INDENT));
+	action_link = GTK_ACTION(gtk_action_new("link", _("Link"), NULL, GTK_STOCK_REDO));
+	action_new = GTK_ACTION(gtk_action_new("new", _("New Note"), NULL, NULL));
+	action_notes = GTK_ACTION(gtk_action_new("open", _("Open"), NULL, GTK_STOCK_OPEN));
+	action_quit = GTK_ACTION(gtk_action_new("quit", _("Close All Notes"), NULL, NULL));
+	action_italic = GTK_ACTION(gtk_toggle_action_new("italic", _("Italic"), NULL, GTK_STOCK_ITALIC));
+	action_strike = GTK_ACTION(gtk_toggle_action_new("strikethrough", _("Strikeout"), NULL, NULL));
+	action_text_style = GTK_ACTION(gtk_action_new("style", _("Style"), NULL, GTK_STOCK_SELECT_FONT));
+	action_zoom_in = GTK_ACTION(gtk_action_new("zoom_in", _("Zoom In"), NULL, GTK_STOCK_ZOOM_IN));
+	action_zoom_out = GTK_ACTION(gtk_action_new("zoom_out", _("Zoom Out"), NULL, GTK_STOCK_ZOOM_OUT));
+	action_find = GTK_ACTION(gtk_action_new("find", _("Find In Note"), NULL, GTK_STOCK_FIND));
 	/* TODO: Use an enum instead of 0 to 3 */
-	action_font_small = GTK_ACTION(gtk_radio_action_new("size:small", "Small", NULL, NULL, 0));
-	action_font_normal = GTK_ACTION(gtk_radio_action_new("size:normal", "Normal", NULL, NULL, 1));
-	action_font_large = GTK_ACTION(gtk_radio_action_new("size:large", "Large", NULL, NULL, 2));
-	action_font_huge = GTK_ACTION(gtk_radio_action_new("size:huge", "Huge", NULL, NULL, 3));
+	action_font_small = GTK_ACTION(gtk_radio_action_new("size:small", _("Small"), NULL, NULL, 0));
+	action_font_normal = GTK_ACTION(gtk_radio_action_new("size:normal", _("Normal"), NULL, NULL, 1));
+	action_font_large = GTK_ACTION(gtk_radio_action_new("size:large", _("Large"), NULL, NULL, 2));
+	action_font_huge = GTK_ACTION(gtk_radio_action_new("size:huge", _("Huge"), NULL, NULL, 3));
 
 	gtk_action_set_sensitive(action_link, FALSE);
 	gtk_action_set_sensitive(action_inc_indent, FALSE);
 	gtk_action_set_sensitive(action_dec_indent, FALSE);
 
 	/* Build the radio action group */
-	gtk_radio_action_set_group(action_font_small, action_group);
-	action_group = gtk_radio_action_get_group(action_font_small);
-	gtk_radio_action_set_group(action_font_normal, action_group);
-	action_group = gtk_radio_action_get_group(action_font_normal);
-	gtk_radio_action_set_group(action_font_large, action_group);
-	action_group = gtk_radio_action_get_group(action_font_large);
-	gtk_radio_action_set_group(action_font_huge, action_group);
-	action_group = gtk_radio_action_get_group(action_font_huge);
-	gtk_radio_action_set_current_value(action_font_normal, 1);
+	gtk_radio_action_set_group(GTK_RADIO_ACTION(action_font_small), radio_group);
+	radio_group = gtk_radio_action_get_group(GTK_RADIO_ACTION(action_font_small));
+	gtk_radio_action_set_group(GTK_RADIO_ACTION(action_font_normal), radio_group);
+	radio_group = gtk_radio_action_get_group(GTK_RADIO_ACTION(action_font_normal));
+	gtk_radio_action_set_group(GTK_RADIO_ACTION(action_font_large), radio_group);
+	radio_group = gtk_radio_action_get_group(GTK_RADIO_ACTION(action_font_large));
+	gtk_radio_action_set_group(GTK_RADIO_ACTION(action_font_huge), radio_group);
+	radio_group = gtk_radio_action_get_group(GTK_RADIO_ACTION(action_font_huge));
+	gtk_radio_action_set_current_value(GTK_RADIO_ACTION(action_font_normal), 1);
 
-	/* Add actions to accelerator group */
-	gtk_action_set_accel_group(action_bold, accel_group);
-	gtk_action_set_accel_group(action_italic, accel_group);
-	gtk_action_set_accel_group(action_fixed, accel_group);
+	action_group = gtk_action_group_new("notewin");
+	/* TODO: This 3 blocks should be one function call. Write a function which executs all three commands */
+	gtk_action_group_add_action_with_accel(action_group, action_bold,      "<Ctrl>b");
+	gtk_action_group_add_action_with_accel(action_group, action_italic,    "<Ctrl>i");
+	gtk_action_group_add_action_with_accel(action_group, action_fixed,     "<Ctrl>m");
+	gtk_action_group_add_action_with_accel(action_group, action_highlight, "<Ctrl>h");
+	gtk_action_group_add_action_with_accel(action_group, action_strike,    "<Ctrl>s");
+	gtk_action_group_add_action_with_accel(action_group, action_quit,      "<Ctrl>q");
+	gtk_action_group_add_action_with_accel(action_group, action_new,       "<Ctrl>n");
+	gtk_action_group_add_action_with_accel(action_group, action_find,      "<Ctrl>f");
+	gtk_action_group_add_action_with_accel(action_group, action_new,       "<Ctrl>n");
+	gtk_action_group_add_action_with_accel(action_group, action_quit,      "<Ctrl>q");
+
+	gtk_action_set_accel_group(action_bold,      accel_group);
+	gtk_action_set_accel_group(action_italic,    accel_group);
+	gtk_action_set_accel_group(action_fixed,     accel_group);
 	gtk_action_set_accel_group(action_highlight, accel_group);
-	gtk_action_set_accel_group(action_strike, accel_group);
-	gtk_action_set_accel_group(action_quit, accel_group);
-	gtk_action_set_accel_group(action_new, accel_group);
-	gtk_action_set_accel_group(action_find, accel_group);
-
-	/* Add accelerator paths to actions */
-	gtk_action_set_accel_path(action_bold,      ACCEL_PATH_STYLE_BOLD);
-	gtk_action_set_accel_path(action_italic,    ACCEL_PATH_STYLE_ITALIC);
-	gtk_action_set_accel_path(action_strike,    ACCEL_PATH_STYLE_STRIKE);
-	gtk_action_set_accel_path(action_highlight, ACCEL_PATH_STYLE_HIGHLIGHT);
-	gtk_action_set_accel_path(action_fixed,     ACCEL_PATH_STYLE_FIXED);
-	gtk_action_set_accel_path(action_quit,      ACCEL_PATH_QUIT);
-	gtk_action_set_accel_path(action_new,       ACCEL_PATH_NEW);
-	gtk_action_set_accel_path(action_find,		ACCEL_PATH_FIND);
-
-	/* Set keybindings for accelerator paths */
-	gtk_accel_map_add_entry(ACCEL_PATH_STYLE_BOLD,      GDK_b, GDK_CONTROL_MASK);
-	gtk_accel_map_add_entry(ACCEL_PATH_STYLE_ITALIC,    GDK_i, GDK_CONTROL_MASK);
-	gtk_accel_map_add_entry(ACCEL_PATH_STYLE_STRIKE,    GDK_s, GDK_CONTROL_MASK);
-	gtk_accel_map_add_entry(ACCEL_PATH_STYLE_HIGHLIGHT, GDK_h, GDK_CONTROL_MASK);
-	gtk_accel_map_add_entry(ACCEL_PATH_STYLE_FIXED,     GDK_m, GDK_CONTROL_MASK);
-	gtk_accel_map_add_entry(ACCEL_PATH_QUIT,            GDK_q, GDK_CONTROL_MASK);
-	gtk_accel_map_add_entry(ACCEL_PATH_NEW,             GDK_n, GDK_CONTROL_MASK);
-	gtk_accel_map_add_entry(ACCEL_PATH_FIND,            GDK_f, GDK_CONTROL_MASK);
-	
+	gtk_action_set_accel_group(action_strike,    accel_group);
+	gtk_action_set_accel_group(action_quit,      accel_group);
+	gtk_action_set_accel_group(action_new,       accel_group);
+	gtk_action_set_accel_group(action_find,      accel_group);
+	gtk_action_set_accel_group(action_new,       accel_group);
+	gtk_action_set_accel_group(action_quit,      accel_group);
 
+	gtk_action_connect_accelerator(action_bold);
+	gtk_action_connect_accelerator(action_italic);
+	gtk_action_connect_accelerator(action_fixed);
+	gtk_action_connect_accelerator(action_highlight);
+	gtk_action_connect_accelerator(action_strike);
+	gtk_action_connect_accelerator(action_find);
+	gtk_action_connect_accelerator(action_new);
+	gtk_action_connect_accelerator(action_quit);
 
 
 	/* FORMAT MENU */
 #ifdef HILDON_HAS_APP_MENU
 	text_style_menu = hildon_app_menu_new();
 
-	menu_bold = gtk_button_new();
-	menu_italic = gtk_button_new();
-	menu_strike = gtk_button_new();
-	menu_highlight = gtk_button_new();
-	menu_fixed = gtk_button_new();
-	menu_bullets = gtk_button_new();
-	menu_inc_indent = gtk_button_new();
-	menu_dec_indent = gtk_button_new();
-
-	menu_font_small = gtk_radio_button_new(NULL);
-	menu_font_normal = gtk_radio_button_new(NULL);
-	menu_font_large = gtk_radio_button_new(NULL);
-	menu_font_huge = gtk_radio_button_new(NULL);
+	menu_bold = gtk_toggle_button_new();
+	menu_italic = gtk_toggle_button_new();
+	menu_strike = gtk_toggle_button_new();
+	menu_highlight = gtk_toggle_button_new();
+	menu_fixed = gtk_toggle_button_new();
+	menu_bullets = gtk_toggle_button_new();
+	menu_font_small = gtk_toggle_button_new();
+	menu_font_normal = gtk_toggle_button_new();
+	menu_font_large = gtk_toggle_button_new();
+	menu_font_huge = gtk_toggle_button_new();
 
 	gtk_action_connect_proxy(action_bold, menu_bold);
 	gtk_action_connect_proxy(action_italic, menu_italic);
@@ -296,37 +304,35 @@ GtkWidget* create_mainwin(Note *note) {
 	gtk_action_connect_proxy(action_highlight, menu_highlight);
 	gtk_action_connect_proxy(action_fixed, menu_fixed);
 	gtk_action_connect_proxy(action_bullets, menu_bullets);
-	gtk_action_connect_proxy(action_inc_indent, menu_inc_indent);
-	gtk_action_connect_proxy(action_dec_indent, menu_dec_indent);
 	gtk_action_connect_proxy(action_font_small, menu_font_small);
 	gtk_action_connect_proxy(action_font_normal, menu_font_normal);
 	gtk_action_connect_proxy(action_font_large, menu_font_large);
 	gtk_action_connect_proxy(action_font_huge, menu_font_huge);
 
-	set_button_label(menu_bold, "<b>Bold</b>");
-	set_button_label(menu_italic, "<i>Italic</i>");
-	set_button_label(menu_strike, "<s>Strikeout</s>");
-	set_button_label(menu_highlight, "<span background=\"yellow\">Highlight</span>");
-	set_button_label(menu_fixed, "<tt>Fixed Width</tt>");
-	set_button_label(menu_font_small, "<span size=\"small\">Small</span>");
-	set_button_label(menu_font_large, "<span size=\"large\">Large</span>");
-	set_button_label(menu_font_huge, "<span size=\"x-large\">Huge</span>");
+	set_item_label(GTK_CONTAINER(menu_bold),       "<b>", _("Bold"), "</b>");
+	set_item_label(GTK_CONTAINER(menu_italic),     "<i>", _("Italic"), "</i>");
+	set_item_label(GTK_CONTAINER(menu_strike),     "<s>", _("Strikeout"), "</s>");
+	set_item_label(GTK_CONTAINER(menu_highlight),  "<span background=\"yellow\">", _("Highlight"), "</span>");
+	set_item_label(GTK_CONTAINER(menu_fixed),      "<tt>", _("Fixed Width"), "</tt>");
+	set_item_label(GTK_CONTAINER(menu_font_small), "<span size=\"small\">", _("Small"), "</span>");
+	set_item_label(GTK_CONTAINER(menu_font_large), "<span size=\"large\">", _("Large"), "</span>");
+	set_item_label(GTK_CONTAINER(menu_font_huge),  "<span size=\"x-large\">", _("Huge"), "</span>");
+
+	hildon_app_menu_append(HILDON_APP_MENU(text_style_menu), GTK_BUTTON(menu_bold));
+	hildon_app_menu_append(HILDON_APP_MENU(text_style_menu), GTK_BUTTON(menu_italic));
+	hildon_app_menu_append(HILDON_APP_MENU(text_style_menu), GTK_BUTTON(menu_strike));
+	hildon_app_menu_append(HILDON_APP_MENU(text_style_menu), GTK_BUTTON(menu_highlight));
+	hildon_app_menu_append(HILDON_APP_MENU(text_style_menu), GTK_BUTTON(menu_fixed));
+	hildon_app_menu_append(HILDON_APP_MENU(text_style_menu), GTK_BUTTON(menu_bullets));
+
+	hildon_app_menu_add_filter(HILDON_APP_MENU(text_style_menu), GTK_BUTTON(menu_font_small));
+	hildon_app_menu_add_filter(HILDON_APP_MENU(text_style_menu), GTK_BUTTON(menu_font_normal));
+	hildon_app_menu_add_filter(HILDON_APP_MENU(text_style_menu), GTK_BUTTON(menu_font_large));
+	hildon_app_menu_add_filter(HILDON_APP_MENU(text_style_menu), GTK_BUTTON(menu_font_huge));
 
-	hildon_app_menu_append(text_style_menu, menu_bold);
-	hildon_app_menu_append(text_style_menu, menu_italic);
-	hildon_app_menu_append(text_style_menu, menu_strike);
-	hildon_app_menu_append(text_style_menu, menu_highlight);
-	hildon_app_menu_append(text_style_menu, menu_fixed);
-	hildon_app_menu_append(text_style_menu, menu_bullets);
-	hildon_app_menu_append(text_style_menu, menu_inc_indent);
-	hildon_app_menu_append(text_style_menu, menu_dec_indent);
-	hildon_app_menu_append(text_style_menu, menu_font_normal);
-	hildon_app_menu_append(text_style_menu, menu_font_small);
-	hildon_app_menu_append(text_style_menu, menu_font_large);
-	hildon_app_menu_append(text_style_menu, menu_font_huge);
 
 #else
-	text_style_menu = GTK_MENU_SHELL(gtk_menu_new());
+	text_style_menu = GTK_WIDGET(gtk_menu_new());
 
 	menu_bold = gtk_action_create_menu_item(action_bold);
 	menu_italic = gtk_action_create_menu_item(action_italic);
@@ -340,30 +346,30 @@ GtkWidget* create_mainwin(Note *note) {
 	menu_font_normal = gtk_action_create_menu_item(action_font_normal);
 	menu_font_large = gtk_action_create_menu_item(action_font_large);
 	menu_font_huge = gtk_action_create_menu_item(action_font_huge);
+	/* TODO: Write a function, which takes GtkActions that contain markup code and that creates the correct widgets */
+	set_item_label(GTK_CONTAINER(menu_bold),       "<b>", _("Bold"), "</b>");
+	set_item_label(GTK_CONTAINER(menu_italic),     "<i>", _("Italic"), "</i>");
+	set_item_label(GTK_CONTAINER(menu_strike),     "<s>", _("Strikeout"), "</s>");
+	set_item_label(GTK_CONTAINER(menu_highlight),  "<span background=\"yellow\">", _("Highlight"), "</span>");
+	set_item_label(GTK_CONTAINER(menu_fixed),      "<tt>", _("Fixed Width"), "</tt>");
+	set_item_label(GTK_CONTAINER(menu_font_small), "<span size=\"small\">", _("Small"), "</span>");
+	set_item_label(GTK_CONTAINER(menu_font_large), "<span size=\"large\">", _("Large"), "</span>");
+	set_item_label(GTK_CONTAINER(menu_font_huge),  "<span size=\"x-large\">", _("Huge"), "</span>");
 
-	set_menu_item_label(GTK_MENU_ITEM(menu_bold),       "<b>Bold</b>");
-	set_menu_item_label(GTK_MENU_ITEM(menu_italic),     "<i>Italic</i>");
-	set_menu_item_label(GTK_MENU_ITEM(menu_strike),     "<s>Strikeout</s>");
-	set_menu_item_label(GTK_MENU_ITEM(menu_highlight),  "<span background=\"yellow\">Highlight</span>");
-	set_menu_item_label(GTK_MENU_ITEM(menu_fixed),      "<tt>Fixed Width</tt>");
-	set_menu_item_label(GTK_MENU_ITEM(menu_font_small), "<span size=\"small\">Small</span>");
-	set_menu_item_label(GTK_MENU_ITEM(menu_font_large), "<span size=\"large\">Large</span>");
-	set_menu_item_label(GTK_MENU_ITEM(menu_font_huge),  "<span size=\"x-large\">Huge</span>");
-
-	gtk_menu_shell_append(text_style_menu, menu_bold);
-	gtk_menu_shell_append(text_style_menu, menu_italic);
-	gtk_menu_shell_append(text_style_menu, menu_strike);
-	gtk_menu_shell_append(text_style_menu, menu_highlight);
-	gtk_menu_shell_append(text_style_menu, menu_fixed);
-	gtk_menu_shell_append(text_style_menu, gtk_separator_menu_item_new());
-	gtk_menu_shell_append(text_style_menu, menu_font_small);
-	gtk_menu_shell_append(text_style_menu, menu_font_normal);
-	gtk_menu_shell_append(text_style_menu, menu_font_large);
-	gtk_menu_shell_append(text_style_menu, menu_font_huge);
-	gtk_menu_shell_append(text_style_menu, gtk_separator_menu_item_new());
-	gtk_menu_shell_append(text_style_menu, menu_bullets);
-	gtk_menu_shell_append(text_style_menu, menu_inc_indent);
-	gtk_menu_shell_append(text_style_menu, menu_dec_indent);
+	gtk_menu_shell_append(GTK_MENU_SHELL(text_style_menu), menu_bold);
+	gtk_menu_shell_append(GTK_MENU_SHELL(text_style_menu), menu_italic);
+	gtk_menu_shell_append(GTK_MENU_SHELL(text_style_menu), menu_strike);
+	gtk_menu_shell_append(GTK_MENU_SHELL(text_style_menu), menu_highlight);
+	gtk_menu_shell_append(GTK_MENU_SHELL(text_style_menu), menu_fixed);
+	gtk_menu_shell_append(GTK_MENU_SHELL(text_style_menu), gtk_separator_menu_item_new());
+	gtk_menu_shell_append(GTK_MENU_SHELL(text_style_menu), menu_font_small);
+	gtk_menu_shell_append(GTK_MENU_SHELL(text_style_menu), menu_font_normal);
+	gtk_menu_shell_append(GTK_MENU_SHELL(text_style_menu), menu_font_large);
+	gtk_menu_shell_append(GTK_MENU_SHELL(text_style_menu), menu_font_huge);
+	gtk_menu_shell_append(GTK_MENU_SHELL(text_style_menu), gtk_separator_menu_item_new());
+	gtk_menu_shell_append(GTK_MENU_SHELL(text_style_menu), menu_bullets);
+	gtk_menu_shell_append(GTK_MENU_SHELL(text_style_menu), menu_inc_indent);
+	gtk_menu_shell_append(GTK_MENU_SHELL(text_style_menu), menu_dec_indent);
 #endif
 
 
@@ -378,16 +384,6 @@ GtkWidget* create_mainwin(Note *note) {
 	gtk_action_connect_proxy(action_new, menu_new);
 	gtk_action_connect_proxy(action_quit, menu_quit);
 
-	/*
-	GtkWidget *one = gtk_button_new_with_label("Something longer");
-	GtkWidget *two = gtk_button_new_with_label("Something very very long");
-	GtkWidget *copy = gtk_button_new_with_label("Highlight");
-	GtkWidget *paste = gtk_button_new_with_label("Strikethrough");
-
-	set_button_label(paste, "<b>Bold</b>");
-	set_button_label(copy, "<i>Italic</i>");
-	*/
-
 	hildon_app_menu_append(HILDON_APP_MENU(main_menu), GTK_BUTTON(menu_new));
 	hildon_app_menu_append(HILDON_APP_MENU(main_menu), GTK_BUTTON(menu_quit));
 
@@ -397,60 +393,59 @@ GtkWidget* create_mainwin(Note *note) {
 	main_menu = gtk_menu_new();
 
 	menu_new = gtk_action_create_menu_item(action_new);
-	menu_text_style = gtk_menu_item_new_with_label("Text Style");
-	gtk_menu_item_set_submenu(menu_text_style, text_style_menu);
+	menu_text_style = gtk_menu_item_new_with_label(_("Text Style"));
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_text_style), text_style_menu);
 	menu_find = gtk_action_create_menu_item(action_find);
 	menu_quit = gtk_action_create_menu_item(action_quit);
 
-	gtk_menu_shell_append(main_menu, menu_new);
-	gtk_menu_shell_append(main_menu, gtk_separator_menu_item_new());
-	gtk_menu_shell_append(main_menu, menu_text_style);
-	gtk_menu_shell_append(main_menu, gtk_separator_menu_item_new());
-	gtk_menu_shell_append(main_menu, menu_find);
-	gtk_menu_shell_append(main_menu, gtk_separator_menu_item_new());
-	gtk_menu_shell_append(main_menu, menu_quit);
+	gtk_menu_shell_append(GTK_MENU_SHELL(main_menu), menu_new);
+	gtk_menu_shell_append(GTK_MENU_SHELL(main_menu), gtk_separator_menu_item_new());
+	gtk_menu_shell_append(GTK_MENU_SHELL(main_menu), menu_text_style);
+	/*
+	gtk_menu_shell_append(GTK_MENU_SHELL(main_menu), gtk_separator_menu_item_new());
+	gtk_menu_shell_append(GTK_MENU_SHELL(main_menu), menu_find);
+	*/
+	gtk_menu_shell_append(GTK_MENU_SHELL(main_menu), gtk_separator_menu_item_new());
+	gtk_menu_shell_append(GTK_MENU_SHELL(main_menu), menu_quit);
 
 	/* Must be at the end of the menu definition */
 	hildon_window_set_menu(HILDON_WINDOW(mainwin), GTK_MENU(main_menu));
 #endif
 
 
-
-
-
 	/* TOOLBAR */
 	toolbar = gtk_toolbar_new();
 
 	/****/
 	button_dec_indent = gtk_action_create_tool_item(action_dec_indent);
-	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), button_dec_indent, -1);
+	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(button_dec_indent), -1);
 	gtk_widget_set_size_request(GTK_WIDGET(button_dec_indent), 85, -1);
 
 	button_inc_indent = gtk_action_create_tool_item(action_inc_indent);
-	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), button_inc_indent, -1);
+	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(button_inc_indent), -1);
 
 	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(), -1);
 
 	button_link = gtk_action_create_tool_item(action_link);
-	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), button_link, -1);
+	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(button_link), -1);
 
 	button_style = gtk_action_create_tool_item(action_text_style);
-	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), button_style, -1);
+	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(button_style), -1);
 
 	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(), -1);
 
 	button_find = gtk_action_create_tool_item(action_find);
-	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), button_find, -1);
+	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(button_find), -1);
 
 	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(), -1);
 
 	button_delete = gtk_action_create_tool_item(action_delete);
-	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), button_delete, -1);
+	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(button_delete), -1);
 
 	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), gtk_separator_tool_item_new(), -1);
 
 	button_notes = gtk_action_create_tool_item(action_notes);
-	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), button_notes, -1);
+	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(button_notes), -1);
 
 
 	gtk_widget_show_all(toolbar);
@@ -466,8 +461,8 @@ GtkWidget* create_mainwin(Note *note) {
 	*/
 
 	/* FIND TOOL BAR */
-	find_bar = hildon_find_toolbar_new("Find:");
-	hildon_window_add_toolbar(mainwin, find_bar);
+	find_bar = hildon_find_toolbar_new(_("Search:"));
+	hildon_window_add_toolbar(HILDON_WINDOW(mainwin), GTK_TOOLBAR(find_bar));
 
 	/* SCROLLED WINDOW */
 #ifdef HILDON_HAS_APP_MENU
@@ -483,7 +478,7 @@ GtkWidget* create_mainwin(Note *note) {
 	/* TEXT VIEW */
 #ifdef HILDON_HAS_APP_MENU
 	textview = hildon_text_view_new();
-	buffer = hildon_text_view_get_buffer(textview);
+	buffer = hildon_text_view_get_buffer(HILDON_TEXT_VIEW(textview));
 #else
 	textview = gtk_text_view_new();
 	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
@@ -525,7 +520,7 @@ GtkWidget* create_mainwin(Note *note) {
 
 	/* Window signals */
 	g_signal_connect(mainwin, "delete-event",
-	        G_CALLBACK(on_window_close_button_clicked),
+	        G_CALLBACK(on_window_delete),
 	        note);
 
 	/* Action signals */
