@@ -18,6 +18,9 @@
 
 
 /* Includes */
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
 #include <hildon/hildon-program.h>
 #include <hildon/hildon-window.h>
 #include <gtk/gtkmain.h>
@@ -35,6 +38,9 @@
 
 #include <stdio.h>
 
+#include <libintl.h>
+#include <locale.h>
+
 #ifndef __USE_XOPEN
 #define __USE_XOPEN
 #endif
@@ -44,24 +50,34 @@
 #include "interface.h"
 #include "callbacks.h"
 #include "note.h"
+#include "app_data.h"
 
 #include "search_window.h"
-
 #include "note_list_store.h"
 
-#include "localisation.h"
-
-#include "../config.h"
-
 
 #define APP_NAME "conboy"
-/*#define APP_VER "0.3.1"*/
 #define APP_SERVICE "de.zwong.conboy"
 #define APP_METHOD "/de/zwong/conboy"
 
 #define MY_ENCODING "utf-8"
 
-
+static void cleanup()
+{
+	AppData *app_data = app_data_get();
+	GtkTreeIter iter;
+	GtkTreeModel *model = GTK_TREE_MODEL(app_data->note_store);
+	gboolean valid = gtk_tree_model_get_iter_first(model, &iter);
+	while (valid) {
+		Note *note;
+		gtk_tree_model_get(model, &iter, NOTE_COLUMN, &note, -1);
+		note_free(note);
+		valid = gtk_tree_model_iter_next(model, &iter);
+	}
+	
+	/* Free AppData */
+	app_data_free();
+}
 
 
 int
@@ -72,9 +88,13 @@ main (int argc, char *argv[])
   osso_context_t *osso_context;
   AppData *app_data;
 
+  /* Init i18n */
+  setlocale(LC_ALL, "");
+  bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
+  bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+  textdomain(GETTEXT_PACKAGE);
+  
   /* Init GTK */
-  /*gtk_init(&argc, &argv);*/
-
 #ifdef HILDON_HAS_APP_MENU
   g_printerr("HILDON INIT \n");
   hildon_gtk_init(&argc, &argv);
@@ -83,7 +103,7 @@ main (int argc, char *argv[])
 #endif
 
   /* Call this to initialize it */
-  app_data = get_app_data();
+  app_data = app_data_get();
 
   /* Initialize maemo application */
   g_printerr("Starting %s, Version %s \n", APP_NAME, VERSION);
@@ -108,6 +128,8 @@ main (int argc, char *argv[])
 
   gtk_main();
 
+  cleanup();
+  
   /* Deinitialize OSSO */
   osso_deinitialize(osso_context);
 
