@@ -1,5 +1,5 @@
 /* This file is part of Conboy.
- * 
+ *
  * Copyright (C) 2009 Cornelius Hald
  *
  * Conboy is free software: you can redistribute it and/or modify
@@ -16,10 +16,17 @@
  * along with Conboy. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <gtk/gtk.h>
 #include <glib.h>
+#include <glib/gstdio.h>
 #include <string.h>
+#include <libintl.h>
 
+#include "app_data.h"
 #include "metadata.h"
 #include "interface.h"
 #include "note.h"
@@ -27,6 +34,7 @@
 #include "deserializer.h"
 #include "note_list_store.h"
 
+#define _(String)gettext(String)
 
 Note* note_create_new()
 {
@@ -44,35 +52,35 @@ Note* note_create_new()
 void note_free(Note *note)
 {
 	g_free(note->ui);
-	/*
 	g_free(note->filename);
 	g_free(note->title);
 	g_free(note->version);
-	*/
 	g_slist_free(note->active_tags);
+	g_free(note);
+	note = NULL;
 }
 
 void note_show_by_title(const char* title)
-{	
+{
 	Note *note;
-	AppData *app_data = get_app_data();
-	
+	AppData *app_data = app_data_get();
+
 	note = note_list_store_find_by_title(app_data->note_store, title);
-	
+
 	if (note == NULL) {
 		note = note_create_new();
-		note->title = title;
+		note->title = g_strdup(title);
 		note_show(note);
 		return;
 	}
-	
+
 	note_show(note);
 }
 
 void note_format_title(GtkTextBuffer *buffer)
-{	
+{
 	GtkTextIter start, end;
-	
+
 	/* Set end iter depending on if we have one or more lines */
 	if (gtk_text_buffer_get_line_count(buffer) == 1) {
 		gtk_text_buffer_get_end_iter(buffer, &end);
@@ -88,10 +96,10 @@ void note_format_title(GtkTextBuffer *buffer)
 
 
 gchar* note_extract_title_from_buffer(GtkTextBuffer *buffer)
-{	
+{
 	GtkTextIter start, end;
 	gchar* title;
-	
+
 	/* Set end iter depending on if we have one or more lines */
 	if (gtk_text_buffer_get_line_count(buffer) == 1) {
 		gtk_text_buffer_get_end_iter(buffer, &end);
@@ -99,7 +107,7 @@ gchar* note_extract_title_from_buffer(GtkTextBuffer *buffer)
 		gtk_text_buffer_get_iter_at_line(buffer, &end, 1);
 		gtk_text_iter_backward_char(&end);
 	}
-	
+
 	gtk_text_buffer_get_start_iter(buffer, &start);
 	title = gtk_text_iter_get_text(&start, &end);
 	return title;
@@ -118,59 +126,61 @@ static
 gboolean is_empty_str(const gchar* str)
 {
 	gchar *tmp;
-	
+
 	if (str == NULL) {
 		return TRUE;
 	}
-	
+
 	tmp = g_strdup(str);
 	g_strstrip(tmp);
-	
+
 	if (strcmp(tmp, "") == 0) {
 		return TRUE;
 	} else {
 		return FALSE;
 	}
-	
+
 	g_free(tmp);
 }
 
 
 void note_save(Note *note)
-{		
+{
 	time_t time_in_s;
 	const gchar* title;
-	const gchar* content;
+	gchar* content;
 	GtkTextIter iter, start, end;
 	GtkTextMark *mark;
 	gint cursor_position;
 	AppData *app_data;
 	GtkTextBuffer *buffer = note->ui->buffer;
-	
+
 	/* If note is empty, don't save */
 	gtk_text_buffer_get_bounds(buffer, &start, &end);
 	content = gtk_text_iter_get_text(&start, &end);
 	if (is_empty_str(content)) {
 		gtk_text_buffer_set_modified(buffer, FALSE);
+		g_free(content);
 		return;
 	}
-	
+	g_free(content);
+
 	/* If buffer is not dirty, don't save */
 	if (!gtk_text_buffer_get_modified(buffer)) {
 		return;
 	}
-	
+
 	/* Get time */
 	time_in_s = time(NULL);
-	
+
 	/* Get cursor position */
 	mark = gtk_text_buffer_get_insert(buffer);
 	gtk_text_buffer_get_iter_at_mark(buffer, &iter, mark);
 	cursor_position = gtk_text_iter_get_offset(&iter);
 
 	/* Get title */
-	title = note_extract_title_from_buffer(buffer); 
-	
+	title = note_extract_title_from_buffer(buffer);
+
 	/* Set meta data */
 	/* We don't change height, width, x and y because we don't use them */
 	note->title = g_strdup(title);
@@ -178,7 +188,7 @@ void note_save(Note *note)
 	note->last_metadata_change_date = time_in_s;
 	note->cursor_position = cursor_position;
 	note->open_on_startup = FALSE;
-	
+
 	if (note->create_date == 0) {
 		note->create_date = time_in_s;
 	}
@@ -194,52 +204,66 @@ void note_save(Note *note)
 	if (note->y == 0) {
 		note->y = 1;
 	}
-	
-	app_data = get_app_data();
-	
+
+	app_data = app_data_get();
+
 	/* Set start and end iterators for serialization */
 	gtk_text_buffer_get_bounds(buffer, &start, &end);
-	
+
 	/* Start serialization */
 	serialize_note(note);
-	
+
 	/* If first save, add to list of all notes */
 	if (!note_list_store_find(app_data->note_store, note)) {
 		note_list_store_add(app_data->note_store, note, NULL);
 	} else {
 		note_list_store_note_changed(app_data->note_store, note);
 	}
-	
+
 	gtk_text_buffer_set_modified(buffer, FALSE);
 }
 
 
-
 void note_close_window(Note *note)
 {
 	HildonProgram *program = hildon_program_get_instance();
 	HildonWindow *window = note->ui->window;
-	AppData *app_data = get_app_data();
+	AppData *app_data = app_data_get();
 	guint count = g_list_length(app_data->open_notes);
-	
+
 	if (count > 1) {
 		hildon_program_remove_window(program, window);
 		gtk_widget_destroy(GTK_WIDGET(window));
+		note->ui = NULL;
 		app_data->open_notes = g_list_remove(app_data->open_notes, note);
 		/* Don't free note, because we reuse this in the menu with the available notes and when reopening */
-		/*note_free(note);*/
-		return;
-	} 
-	
-	g_printerr("####################### QUIT ###################");
-	gtk_main_quit();
+	} else {
+		g_printerr("####################### QUIT ###################\n");
+		gtk_main_quit();
+	}
 }
 
 
+void note_delete(Note *note)
+{
+	AppData *app_data = app_data_get();
+
+	/* Delete file */
+	if (g_unlink(note->filename) == -1) {
+		g_printerr("ERROR: The file %s could not be deleted \n", note->filename);
+	}
+
+	/* Remove from list store */
+	note_list_store_remove(app_data->note_store, note);
+
+	/* Free memory */
+	/*note_free(note);*/
+}
+
 
 gboolean note_is_open(Note *note)
-{	
-	AppData *app_data = get_app_data();
+{
+	AppData *app_data = app_data_get();
 	GList *element = g_list_find(app_data->open_notes, note);
 	if (element == NULL) {
 		return FALSE;
@@ -250,7 +274,7 @@ gboolean note_is_open(Note *note)
 
 gboolean note_exists(Note *note)
 {
-	AppData *app_data = get_app_data();
+	AppData *app_data = app_data_get();
 	Note *element = note_list_store_find(app_data->note_store, note);
 	if (element == NULL) {
 		return FALSE;
@@ -270,74 +294,74 @@ void note_set_focus(Note *note)
 
 void note_show(Note *note)
 {
-	AppData *app_data = get_app_data();
+	AppData *app_data = app_data_get();
 	GtkTextBuffer *buffer;
 	GtkWindow *window;
-	
+
 	/* If the note it already open, we bring this note to the foreground and return */
 	if (note_is_open(note)) {
 		note_set_focus(note);
 		return;
 	}
-	
+
 	create_mainwin(note);
-	
+
 	buffer = note->ui->buffer;
 	window = GTK_WINDOW(note->ui->window);
-	
+
 	/* Set window width/height, otherwise scroll to cursor doesn't work correctly */
 	gtk_window_set_default_size(window, 720, 420);
-	
+
 	/* Set to fullscreen or not */
 	if (app_data->fullscreen) {
 		gtk_window_fullscreen(GTK_WINDOW(note->ui->window));
 	} else {
 		gtk_window_unfullscreen(GTK_WINDOW(note->ui->window));
 	}
-	
+
 	hildon_program_add_window(app_data->program, HILDON_WINDOW(window));
-	
+
 	/* Block signals on TextBuffer until we are done with initializing the content. This is to prevent saves etc. */
 	g_signal_handlers_block_matched(buffer, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, note);
-	
+
 	if (note_exists(note)) {
 		note_show_existing(note);
 	} else {
 		note_show_new(note);
 	}
-	
+
 	app_data->open_notes = g_list_append(app_data->open_notes, note);
-	
+
 	/* Format note title and update window title */
 	note_format_title(buffer);
 	note_set_window_title_from_buffer(window, buffer); /* Replace this. And use note->title instead */
-	
+
 	gtk_widget_show(GTK_WIDGET(window));
-	
+
 	gtk_text_buffer_set_modified(buffer, FALSE);
-	
+
 	/* unblock signals */
 	g_signal_handlers_unblock_matched(buffer, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, note);
 }
 
 void note_show_new(Note *note)
 {
-	AppData *app_data = get_app_data();
+	AppData *app_data = app_data_get();
 	GtkTextBuffer *buffer = note->ui->buffer;
 	GtkTextIter start, end;
 	const gchar *content;
 	gint notes_count;
-	
-	note->filename = note_get_new_filename();
-	
+
+	note->filename = (gchar*)note_get_new_filename();
+
 	if (note->title == NULL) {
-		notes_count = note_list_store_get_length(app_data->note_store); /*g_list_length(app_data->all_notes);*/
-		note->title = g_strdup_printf("New Note %i", notes_count);
+		notes_count = note_list_store_get_length(app_data->note_store);
+		note->title = g_strdup_printf(_("New Note %i"), notes_count);
 	}
-	
-	content = g_strconcat(note->title, "\n\n", "Describe your new note here.", NULL);
+
+	content = g_strconcat(note->title, "\n\n", _("Describe your new note here."), NULL);
 	gtk_text_buffer_set_text(buffer, content, -1);
-	
+
 	/* Select the text */
 	gtk_text_buffer_get_iter_at_line(buffer, &start, 2);
 	gtk_text_buffer_get_end_iter(buffer, &end);
@@ -345,20 +369,20 @@ void note_show_new(Note *note)
 }
 
 void note_show_existing(Note *note)
-{	
+{
 	GtkTextBuffer *buffer = note->ui->buffer;
 	GtkTextIter iter;
-	
+
 	/* iter defines where to start with inserting */
 	gtk_text_buffer_get_start_iter(buffer, &iter);
-	
+
 	/* start deserialization */
 	deserialize_note(note);
 
 	/* Set cursor possition */
 	gtk_text_buffer_get_iter_at_offset(buffer, &iter, note->cursor_position);
 	gtk_text_buffer_place_cursor(buffer, &iter);
-	
+
 	/* Scroll to cursor position */
 	gtk_text_view_scroll_to_mark(note->ui->view, gtk_text_buffer_get_insert(buffer), 0.0, TRUE, 0.0, 0.5);
 }
@@ -379,7 +403,7 @@ void note_remove_active_tag_by_name(Note *note, const gchar *tag_name)
 void note_add_active_tag(Note *note, GtkTextTag *tag)
 {
 	GSList *tags;
-	
+
 	if (note == NULL) {
 		g_printerr("ERROR: note_add_active_tag: note is NULL\n");
 		return;
@@ -388,7 +412,7 @@ void note_add_active_tag(Note *note, GtkTextTag *tag)
 		g_printerr("ERROR: note_add_active_tag: tag is NULL\n");
 		return;
 	}
-	
+
 	tags = note->active_tags;
 	while (tags != NULL) {
 		if (strcmp(((GtkTextTag*)tags->data)->name, tag->name) == 0) {
