@@ -1,3 +1,25 @@
+/* This file is part of Conboy.
+ * 
+ * Copyright (C) 2009 Cornelius Hald
+ *
+ * Conboy is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Conboy is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Conboy. If not, see <http://www.gnu.org/licenses/>.
+ */
+
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <libxml/encoding.h>
 #include <libxml/xmlwriter.h>
 #include <string.h>
@@ -206,7 +228,7 @@ void write_content(xmlTextWriter *writer, Note *note)
 	int rc = 0;
 	GtkTextBuffer *buffer;
 	GtkTextIter start, end;
-	GtkTextIter *old_iter, *iter;
+	GtkTextIter old_iter, iter;
 	gchar *text;
 	GSList *start_tags = NULL, *end_tags = NULL;
 	
@@ -227,13 +249,13 @@ void write_content(xmlTextWriter *writer, Note *note)
 	buffer = note->ui->buffer;
 	
 	gtk_text_buffer_get_bounds(buffer, &start, &end);
-	iter = gtk_text_iter_copy(&start);
-	old_iter = gtk_text_iter_copy(&start);
+	iter = start;
+	old_iter = start;
 	
-	while (gtk_text_iter_compare(iter, &end) <= 0) {
+	while (gtk_text_iter_compare(&iter, &end) <= 0) {
 		
-		start_tags = gtk_text_iter_get_toggled_tags(iter, TRUE);
-		end_tags   = gtk_text_iter_get_toggled_tags(iter, FALSE);
+		start_tags = gtk_text_iter_get_toggled_tags(&iter, TRUE);
+		end_tags   = gtk_text_iter_get_toggled_tags(&iter, FALSE);
 		
 		/* Write end tags */
 		g_slist_foreach(end_tags, (GFunc)write_end_element, writer);
@@ -246,16 +268,15 @@ void write_content(xmlTextWriter *writer, Note *note)
 		
 		/* Move iter */
 		/* Remember position and set iter to next toggle */
-		gtk_text_iter_free(old_iter);
-		old_iter = gtk_text_iter_copy(iter);
+		old_iter = iter;
 		
-		if (gtk_text_iter_compare(iter, &end) >= 0) {
+		if (gtk_text_iter_compare(&iter, &end) >= 0) {
 			break;
 		}
-		gtk_text_iter_forward_to_tag_toggle(iter, NULL);
+		gtk_text_iter_forward_to_tag_toggle(&iter, NULL);
 		
 		/* Write text */
-		text = gtk_text_iter_get_text(old_iter, iter);
+		text = gtk_text_iter_get_text(&old_iter, &iter);
 		write_text(text, writer);
 	}
 	
